# encoding: utf-8
require 'rubyXL'
class Admin::AnnouncementsController < OrbitAdminController
  include Admin::AnnouncementsHelper
  before_action ->(module_app = @app_title) { set_variables module_app }
  before_action :set_bulletin, only: [:edit, :destroy]
  before_action :load_access_level, :load_settings

  def initialize
    super
    @app_title = "announcement"
  end

  def index
    Bulletin.remove_expired_status
    @tags = @module_app.tags
    @table_fields = [:status, :category, :title, :start_date, :end_date, :last_modified]
    @current_user = current_user
    if AnnouncementSetting.first.is_display_edit_only && !current_user.is_admin? && !current_user.is_manager?(@module_app)
      current_user_is_sub_manager = !current_user.is_manager?(@module_app) && (current_user.is_sub_manager?(@module_app) || current_user.is_sub_manager_with_role?(@module_app)) rescue false
      if current_user_is_sub_manager
        @categories = current_user.approved_categories.select{|c| c.module_app_id == @module_app.id} rescue []
        @filter_fields = filter_fields(@categories, @tags)
        @bulletins = Bulletin.where(:create_user_id=>current_user.id,:title.ne => "",:is_preview.in=>[false,nil])
                                        .order_by(sort)
                                        .with_categories(filters("category"))
                                        .with_tags(filters("tag"))
                                        .with_status(filters("status"))
      else
        @bulletins = Bulletin.where(:uid=>nil)
        @categories = @module_app.categories.enabled
        @filter_fields = filter_fields(@categories, @tags)
      end
    else
      @categories = @module_app.categories.enabled
      @filter_fields = filter_fields(@categories, @tags)
      @bulletins = Bulletin.where(:title.ne => "",:is_preview.in=>[false,nil])
                                        .order_by(sort)
                                        .with_categories(filters("category"))
                                        .with_tags(filters("tag"))
                                        .with_status(filters("status"))
    end
    @bulletins = search_data(@bulletins,[:title]).page(params[:page]).per(10)
    
    if request.xhr?
      render :partial => "index"
    end
  end

  def feed
    @table_feed_fields = ["announcement.feed_name", :tags, "announcement.rssfeed", "announcement.jsonfeed"]
    @feeds = BulletinFeed.all.asc(:created_at)
    
  end
  def generate_iframe_url
    iframe_params = params.require(:iframe).permit!
    uids = iframe_params['member_ids'].to_a.map{|m_id| MemberProfile.find(m_id).uid rescue nil}.select{|uid| !uid.nil?}
    url_params = iframe_params.except(:member_ids)
    url_params['uids'] = uids if uids != []
    render :text => '/xhr/panel/announcement/widget/sync_data?'+url_params.to_param
  end
  def settings
    @setting = @announcement_setting
    roles = Role.all
    @sorted_members = roles.inject({}) do |members,role|
      members_for_role =  role.member_profiles.select{|m| (m.user.nil? ? false : m.user.approved)}
      members[role] = members_for_role
      members
    end
    @sorted_members['no_role'] = MemberProfile.any_in(:role_ids=>[nil,[]]).select{|m| (m.user.nil? ? false : m.user.approved)}
    @unapproved_members = User.where(:approved => false).map{|u| u.member_profile}
  end

  def import
  end

  def excel_format
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="announcement_import_format.xlsx"'
      }
    end
  end

  def export_excel
    @announcements = Bulletin.all.desc(:created_at)
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="announcement_export.xlsx"'
      }
    end
  end

  def import_from_xml
    download_tmp_xml params["import_xml"]
    import_from_tmp_xml File.read(File.join(Rails.root, "tmp", "ann_cc_ntu.xml"))
    redirect_to admin_announcements_path
  end

  def import
  end


  def import_from_wp
    import_from_wordpress params["import_xml"].tempfile
    redirect_to admin_announcements_path
  end 

  def importanns
    workbook = RubyXL::Parser.parse(params["import_file"].tempfile)
    categories = @module_app.categories.asc(:created_at).to_a
    tags = @module_app.tags.asc(:created_at).to_a
    sheet = workbook[0]
    if sheet.count <= 503
      sheet.each_with_index do |row, i|
        next if i < 3
        v = row.cells.first.value
        next if v == "" || v.nil?
        import_this_announcement(row, categories, tags)
      end
      redirect_to admin_announcements_path
    else
      redirect_to admin_announcements_path(:error => "1")
    end
  end

  def createsettings
    setting = AnnouncementSetting.new(settings_params)
    setting.save
    redirect_to admin_announcement_settings_path
  end
  
  def updatesettings
    setting = @announcement_setting
    ids = params['announcement_setting']['anns_status_settings'].to_a.collect do |i,v|
      v['_id']
    end.compact
    AnnsStatusSetting.where(:id.nin=>ids).destroy
    setting.update_attributes(settings_params)
    setting.save
    redirect_to admin_announcement_settings_path
  end

  def feedform
    if params[:type] == "new"
      @announcement_feed = BulletinFeed.new
      render :partial => "feed_form"
    else params[:type] == "edit"
      @announcement_feed = BulletinFeed.find(params[:id])
      render :partial => "edit_feed_form"
    end
  end

  def createfeed
    announcement_feed = BulletinFeed.new(feed_params)
    announcement_feed.save
    feeds = BulletinFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end

  def updatefeed
    ann_feed  = BulletinFeed.find(params[:id])
    ann_feed.update_attributes(feed_params)
    ann_feed.save
    feeds = BulletinFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end

  def deletefeed
    ann_feed  = BulletinFeed.find(params[:id])
    ann_feed.destroy
    feeds = BulletinFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end

  def new
    @tags = @module_app.tags
    @statuses = []
    @bulletin = Bulletin.new
    @bulletin.email_sentdate = Time.now
    @reach_limit = @bulletin.check_status_limit(current_user,true)
    if defined? Calendar
      categories = user_authenticated_categories rescue ['all']
      if categories.first == "all"
        @calendar_categories = CalendarType.all
      else
        @calendar_categories = CalendarType.where(:category_id.in => categories) rescue []
      end
    end
  end

  def create
    bps = bulletin_params
    bulletin = Bulletin.new(bps)
    if !bps['bulletin_links_attributes'].nil?
      bps['bulletin_links_attributes'].each do |idx,link|
        bps['bulletin_links_attributes'].delete(idx.to_s) if link['url'].blank?
      end
    end
    if((!AnnouncementSetting.first.only_manager_can_edit_status) || (AnnouncementSetting.first.only_manager_can_edit_status && (@current_user.is_admin? || @current_user.is_manager?(@module_app))) )
      if bps[:is_top] == "1" && !AnnouncementSetting.check_limit_for_user(bulletin.create_user_id, bulletin.id)
        bps[:is_top] = "0"
        bps[:top_end_date] = nil
      end
    else
      bps[:is_top] = false
      bps[:is_hot] = false
      bps[:is_hidden] = false
    end

    if !defined?(Calendar).nil?
      if bps[:add_to_calendar] == '0' && !bps[:event_id].blank?
        Event.find(bps[:event_id]).destroy rescue nil
        bps[:event_id] = nil
      elsif bps[:add_to_calendar] == '1'
        event = Event.find(bps[:event_id]) rescue Event.new(create_user_id: current_user.id)
        e_start = bps[:calendar_start_date].blank? ? bps[:postdate] : bps[:calendar_start_date]
        e_start = Time.now.to_datetime if e_start.blank?
        e_end = bps[:calendar_end_date].blank? ? bps[:deadline] : bps[:calendar_end_date]
        e_end = Time.now.to_datetime + 1.year if e_end.blank?
        event.update_attributes(bulletin_id: bulletin.id,start: e_start,end: e_end,update_user_id: current_user.id,all_day: bps[:calendar_all_day],calendar_type_id: bps[:calendar_type_id],title: bps[:title_translations][I18n.locale],note: bps[:subtitle_translations][I18n.locale])
        bps[:event_id] = event.id
      end
    end
    bulletin.create_user_id = current_user.id
    bulletin.update_user_id = current_user.id
    if AnnouncementSetting.is_pro?
      if user_can_approve?
        bulletin.approved = true
      else
        send_notification_mail_to_managers(bulletin,"approval",I18n.locale)
      end
    else
      bulletin.approved = true
    end
    
    bulletin.save
    build_email(bulletin,I18n.locale)
    redirect_to params['referer_url']
  end

  def approve_bulletin
    id = params[:id]
    bulletin = Bulletin.find(id)
    if params["approved"] == "true"
      bulletin.approved = true
      bulletin.rejected = false
      bulletin.reapproval = false
    else
      bulletin.rejected = true
      bulletin.reapproval = false
      bulletin.rejection_reason = params["reason"]
      send_rejection_email(bulletin,I18n.locale)
    end
    bulletin.save
    redirect_to admin_announcements_path
  end

  def edit
    if can_edit_or_delete?(@bulletin)
      @reach_limit = @bulletin.check_status_limit(current_user,true)
      @tags = @module_app.tags
      @categories = @module_app.categories.enabled
      if defined? Calendar
        categories = user_authenticated_categories rescue ['all']
        if categories.first == "all"
          @calendar_categories = CalendarType.all
        else
          @calendar_categories = CalendarType.where(:category_id.in => categories) rescue []
        end
      end
      @statuses = []
      @bulletin.email_sentdate = Time.now if @bulletin.email_sent == false
    else
      render_401
    end
  end

  def update
    uid = params[:id].split('-').last
    bulletin = Bulletin.find_by(:uid=>uid)
    bps = bulletin_params
    bps[:tags] = bps[:tags].blank? ? [] : bps[:tags]
    bps[:email_member_ids] = bps[:email_member_ids].blank? ? [] : bps[:email_member_ids]

    if !bps['bulletin_links_attributes'].nil?
      bps['bulletin_links_attributes'].each do |idx,link|
        bps['bulletin_links_attributes'].delete(idx.to_s) if link['url'].blank?
      end
    end
    if((!AnnouncementSetting.first.only_manager_can_edit_status) || (AnnouncementSetting.first.only_manager_can_edit_status && (@current_user.is_admin? || @current_user.is_manager?(@module_app))) )
      if bps[:is_top] == "1" && !AnnouncementSetting.check_limit_for_user(bulletin.create_user_id, bulletin.id)
        bps[:is_top] = "0"
        bps[:top_end_date] = nil
      end
    else
      bps[:is_top] = bulletin.is_top
      bps[:is_hot] = bulletin.is_hot
      bps[:is_hidden] = bulletin.is_hidden
    end
    if !defined?(Calendar).nil?
      if bps[:add_to_calendar] == '0' && !bps[:event_id].blank?
        Event.find(bps[:event_id]).destroy rescue nil
        bps[:event_id] = nil
      elsif bps[:add_to_calendar] == '1'
        event = Event.find(bps[:event_id]) rescue Event.new(create_user_id: current_user.id)
        e_start = bps[:calendar_start_date].blank? ? bps[:postdate] : bps[:calendar_start_date]
        e_start = Time.now.to_datetime if e_start.blank?
        e_end = bps[:calendar_end_date].blank? ? bps[:deadline] : bps[:calendar_end_date]
        e_end = Time.now.to_datetime + 1.year if e_end.blank?
        event.update_attributes(bulletin_id: bulletin.id,start: e_start,end: e_end,update_user_id: current_user.id,all_day: bps[:calendar_all_day],calendar_type_id: bps[:calendar_type_id],title: bps[:title_translations][I18n.locale],note: bps[:subtitle_translations][I18n.locale])
        bps[:event_id] = event.id
      end
    end
    bulletin.update_attributes(bps)
    bulletin.update_user_id = current_user.id
    if bulletin.rejected
      bulletin.reapproval = true
      bulletin.save
      send_notification_mail_to_managers(bulletin,"reapproval",I18n.locale)
    else
      bulletin.save
    end
    build_email(bulletin,I18n.locale)
    now_bulletin_page = Bulletin.where(:title.ne => "",:is_preview.in=>[false,nil])
                              .order_by(sort).map(&:id).map.with_index.select{|v,i| v==bulletin.id}[0][1] rescue nil
    now_bulletin_page = now_bulletin_page.nil? ? 0 : ((now_bulletin_page+1).to_f/10).ceil
    redirect_to "/zh_tw/admin/announcements?page=#{now_bulletin_page}"
  end

  def destroy
    @bulletin.destroy
    redirect_to "/admin/announcements"
  end

  def delete
    if params[:ids]
      Bulletin.any_in(:uid => params[:ids]).destroy_all
    end
    redirect_to "/admin/announcements"
  end

  def preview
    if params['preview_type'].eql?('edit')
      bulletin_data = bulletin_params
      org_bulletin = Bulletin.find(params['bulletin_id'])
      bulletin = org_bulletin.clone
      bulletin.generate_uid
      bulletin.bulletin_files = []
      bulletin.bulletin_links = []
      if bulletin_data['image'].blank?
        bulletin.image = org_bulletin.image
      end

      if !bulletin_data['bulletin_files_attributes'].blank?
        bulletin_data['bulletin_files_attributes'].each do |key, bulletin_file|
          next if !bulletin_file['_destroy'].blank?
          file = nil
          if bulletin_file['id'].blank?
            file = BulletinFile.new(bulletin_file)
            file.bulletin_id = bulletin.id
            file.save
          else
            org_file = BulletinFile.find(bulletin_file['id'])
            file = org_file.clone
            file.bulletin_id = bulletin.id
            file.file = org_file.file
            bulletin_file.delete('id')
            bulletin_file.delete('_destroy')
            file.update_attributes(bulletin_file)
          end

          file.save
          bulletin.bulletin_files << file
        end
      end

      if !bulletin_data['bulletin_links_attributes'].blank?
        bulletin_data['bulletin_links_attributes'].each do |key, bulletin_link|
          next if !bulletin_link['_destroy'].blank?

          if bulletin_link['id'].blank?
            link = BulletinLink.new(bulletin_link)
            link.bulletin_id = bulletin.id
          else
            link = BulletinLink.find(bulletin_link['id']).clone
            link.bulletin_id = bulletin.id
            bulletin_link.delete('id')
            bulletin_link.delete('_destroy')
            link.update_attributes(bulletin_link)
          end

          link.save
          bulletin.bulletin_links << link
        end
      end

      bulletin_data.delete('bulletin_files_attributes')
      bulletin_data.delete('bulletin_links_attributes')
      bulletin.update_attributes(bulletin_data)
    else
      bulletin = Bulletin.new(bulletin_params)
    end
    
    bulletin.is_preview = true
    bulletin.save
    render :text=>page_for_bulletin(bulletin) + "?preview=true"
  end

  def destroy_preview
    bulletin = Bulletin.find_by(:uid=>params['uid'])
    if bulletin.is_preview
      bulletin.destroy
    end
    render :json=>{'destroy'=>bulletin.id.to_s}
  end

  def build_email(bulletin,locale)
    if bulletin.email_sent and !bulletin.email_addresses.blank?
      if bulletin.email.nil?
        email = Email.new
        email.save
        email.deliver rescue nil
        bulletin.email_id = email.id
        bulletin.save
      end

      is_sent = bulletin.email.is_sent
      is_sent = !params[:resend_mail].eql?("true") if !params[:resend_mail].blank?
      doc = Nokogiri::HTML(bulletin.title_translations[locale])
      title = doc.text.empty? ? 'no content' : doc.text
        
      bulletin.email.update_attributes(
        :create_user=>current_user,
        :mail_sentdate=>bulletin.email_sentdate,
        :module_app=>@module_app,
        :mail_lang => locale,
        :mail_to=>bulletin.email_addresses,
        :mail_subject=>title,
        :template=>'announcements/email',
        :template_data=>{
          "host" => request.host_with_port,
          "title" => title,
          "url" => page_for_bulletin(bulletin)
        },
        :is_sent=>is_sent
      )
      bulletin.email.deliver
    else
      bulletin.email.destroy if !bulletin.email.nil?
    end
  end

  private

  def load_settings
    @announcement_setting = AnnouncementSetting.first rescue nil
    if @announcement_setting.nil?
      @announcement_setting = AnnouncementSetting.create
    end
  end

  def set_bulletin
    @bulletin = Bulletin.find(params[:id])
  end

  def bulletin_params
    params[:bulletin][:email_sent] = params[:bulletin][:email_sent].nil? ? 0 : params[:bulletin][:email_sent]
    params.require(:bulletin).permit!
  end

  def feed_params
    params.require(:bulletin_feed).permit!
  end

  def settings_params
    params.require(:announcement_setting).permit!
  end
end
