class AnnouncementsController < ApplicationController
  include AnnouncementsHelper
  def index
    Bulletin.remove_expired_status
    sorted,total_pages = get_sorted_annc
    anns = sorted.collect do |a|
      if a["source-site"].blank?
        statuses = a.statuses_with_classname.collect do |status|
          {
            "status" => status["name"],
            "status-class" => "status-#{status['classname']}"
          }
        end
        locale = OrbitHelper.get_site_locale.to_s
        files = a.bulletin_files.map{|file|  { "file_url" =>  file.file.url, "file_title" => (file.title.blank? ? File.basename(file.file.path) : file.title rescue '') } if file.enabled_for?(locale) } rescue []
        files.delete(nil)
        links = a.bulletin_links.map{|link|  { "link_url" => link.url, "link_title" => (link.title.blank? ? link.url : link.title) } } rescue []
        author = User.find(a.create_user_id).member_profile.name rescue ""
        desc = a.image_description
        desc = (desc.blank? ? "announcement image" : desc)
        link_to_show = a.is_external_link ? a.external_link :  OrbitHelper.url_to_show(a.to_param)
        target = a.is_external_link ? "_blank" : "_self"
        doc = Nokogiri::HTML(a.title)
        title = doc.text.empty? ? 'no content' : doc.text
        {
          "department" => author,
          "bulletin_links" => links,
          "bulletin_files" => files,
          "title" => a.title,
          "source-site" => "",
          "source-site-title" => "",
          "source-site-link" => "",
          "subtitle" => a.subtitle,
          "statuses" => statuses,
          "category" => a.category.title,
          "postdate" => a.postdate,
          "author" => author,
          "is_top" => (a.is_top? ? 1 : 0),
          "link_to_show" => link_to_show+"\" title=\"#{title}\"",
          "target" => target,
          "img_src" => a.image.thumb.url || "/assets/announcement-default.jpg",
          "img_description" => desc,
          "more" => t(:more_plus),
          "view_count" => a.view_count
        }
      else
        a
      end
    end
    #If no data , hide title&table
    if sorted.count == 0
      display = "hide"
    end
    # anns = anns.concat(feeds_anns)
    # total_pages = announcements.total_pages


    {
      "announcements" => anns,
      "extras" => {
        "widget-title" =>t('announcement.announcement'),
        "title-head" => t('announcement.table.title'),
        "date-head" => t('announcement.table.date'),
        "status-head" => t('announcement.table.status'),
        "author-head" => t('announcement.table.author'),
        "subtitle-head" => t('announcement.table.sub_title'),
        "category-head" => t('announcement.table.category'),
        "link-head" => t('announcement.table.link'),
        "file-head" => t('announcement.table.file'),
        "view-count-head" => t('announcement.table.view_count'),
        "display" => display,
        "department-head" => t('announcement.table.department')
      },
      "total_pages" => total_pages
    }

  end

  def random_announcement_widget
    pack_data(true)
  end

  def widget
    pack_data()
  end

  def tag_cloud
    ma = ModuleApp.where(:key => "announcement").first
    temp = []
    ma.tags.each do |tag|
      t1 = tag.taggings.collect{|t| t.taggable_id.to_s}
      count = Bulletin.where(:id.in => t1).can_display_and_sorted.count
      temp << {
        "tag-name" => tag.name,
        "count" => count,
        "tag-url" => OrbitHelper.widget_more_url + "?tags[]=" + tag.id.to_s
      }
    end
    max = temp.max_by{|t| t["count"]}["count"]
    tags = []
    temp.each do |tag|
      if tag["count"] > 0
        percent = (tag["count"] * 100) / max
        font_size = ((percent / 10).round) + 16
        tag["font-size"] = font_size
        tags << tag
      end
    end
    {
      "tags" => tags,
      "extras" => {}
    }
  end

  def pack_data(is_random=false)
    tags = OrbitHelper.widget_tags || []
    cats = OrbitHelper.widget_categories || []
    subpart = OrbitHelper.get_current_widget
    anns_cache = AnnsCache.where(parent_id: subpart.id.to_s,locale: I18n.locale.to_s)
    widget_data_count = OrbitHelper.widget_data_count
    set_image_version_for_widget()
    devide_flag = (!(defined? SiteFeed).nil?)
    if anns_cache.count != 1 || is_random
      page = Page.where(:module => "announcement").first rescue nil
      Bulletin.remove_expired_status
      uid = OrbitHelper.params[:uid] rescue ""
      sorted_anns = Bulletin.where(:title.nin => ["",nil],:is_preview.in=>[false,nil], :uid.ne => uid)
                      .can_display_and_sorted.is_approved
                        .filter_by_widget_categories(cats,false).filter_by_tags(tags)
      if !is_random
        sorted_anns = sorted_anns.limit(widget_data_count)
        if anns_cache.count > 1
          anns_cache.destroy
        end
        if devide_flag
          now_anns = sorted_anns.to_a
          top_anns = now_anns.select{|v| v.is_top}.map{|v| data_to_human_type(v)}
          not_top_anns = now_anns.select{|v| !v.is_top}.map{|v| data_to_human_type(v)}
          AnnsCache.create(parent_id: subpart.id.to_s,locale: I18n.locale.to_s,filter_result: {top: top_anns,not_top: not_top_anns})
        else
          anns = sorted_anns.map{|v| data_to_human_type(v)}
          AnnsCache.create(parent_id: subpart.id.to_s,locale: I18n.locale.to_s,filter_result: anns)
        end
      else
        if devide_flag
          anns = sorted_anns.sample(widget_data_count)
          top_anns = anns.select{|v| v.is_top}.map{|v| data_to_human_type(v)}
          not_top_anns = anns.select{|v| !v.is_top}.map{|v| data_to_human_type(v)}
        else
          anns = sorted_anns.sample(widget_data_count).map{|v| data_to_human_type(v)}
        end
      end
    elsif devide_flag
      now_anns = anns_cache.first.filter_result
      top_anns = now_anns[:top]
      not_top_anns = now_anns[:not_top]
    else
      anns = anns_cache.first.filter_result
    end
    if devide_flag
      rest_count = widget_data_count - top_anns.count
      if rest_count <= 0
        anns = top_anns
      else
        feeds_anns = get_feed_announcements("widget")
        top_anns = top_anns + feeds_anns.select{|v| v['is_top']}
        top_anns = top_anns.sort{|v1,v2| v2["postdate"]<=>v1["postdate"]}
        rest_all_anns = feeds_anns.select{|v| v['is_top'] != true} + not_top_anns.take(rest_count)
        rest_anns = rest_all_anns.sort{|v1,v2| v2["postdate"]<=>v1["postdate"]}.take(rest_count)
        anns = (top_anns + rest_anns).take(widget_data_count)
      end
    end
    mp = (anns[0]["img_src"] rescue "")
    mpd = (anns[0]["img_description"] rescue "")
    {
      "announcements" => anns,
      "extras" => {
        "more_url"=>OrbitHelper.widget_more_url,
        "main_picture" => mp,
        "main_picture_description" => mpd,
        "title-head" => t('announcement.table.title'),
        "date-head" => t('announcement.table.date'),
        "author-head" => t('announcement.table.author'),
        "status-head" => t('announcement.table.status'),
        "subtitle-head" => t('announcement.table.sub_title'),
        "category-head" => t('announcement.table.category'),
        "link-head" => t('announcement.table.link'),
        "file-head" => t('announcement.table.file'),
        "read_more" => ("/#{I18n.locale.to_s + page.url}" rescue "")
      }
    }
  end

  def show_local_announcement(uid, is_preview)
    locale = OrbitHelper.get_site_locale.to_s
    if is_preview
      announcement = Bulletin.where(:uid => uid).first
    else
      announcement = Bulletin.can_display_and_sorted.where(:uid => uid).first
    end
    announcement = Bulletin.where(:uid => uid).first if announcement.nil?
    url_to_edit = OrbitHelper.user_can_edit?(announcement) ? "/admin/announcements/#{announcement.id.to_s}/edit" : ""

    access_level = OrbitHelper.user_access_level?

    if !announcement.approved && (access_level != "manager" && access_level != "admin")
      if AnnouncementSetting.is_pro?
        if !(access_level == "sub_manager" && AnnouncementSetting.first.approvers.include?(OrbitHelper.current_user.id.to_s))
          return {}
        end
      elsif access_level != "sub_manager"
        return {}
      end
    end

    return {} if announcement.category.disable

    tags = announcement.tags.map{|tag|  {
      "tag" => tag.name ,
      "url" => OrbitHelper.page_for_tag(tag)
    } } rescue []
    files = announcement.bulletin_files.map{|file|  { "file_url" =>  file.file.url, "file_title" => (file.title.blank? ? URI.unescape(File.basename(file.file.path)) : file.title rescue '') } if file.enabled_for?(locale) } rescue []
    files.delete(nil)
    files.each do |file|
      if file["file_url"] =="" || file["file_url"] == nil
        files.delete(file)
      end
    end
    links = announcement.bulletin_links.map{|link|  { "link_url" => link.url, "link_title" => (link.title.blank? ? link.url : link.title) } } rescue []
    update_user = announcement.update_user.member_profile.name rescue ""
    desc = announcement.image_description
    desc = (desc.nil? || desc == "" ? "announcement image" : desc)

    request = OrbitHelper.request
    meta_desc = announcement.subtitle.nil? || announcement.subtitle == ""  ? announcement.text[0..200] : announcement.subtitle
    OrbitHelper.render_meta_tags([{"property" => "og:title", "content" => announcement.title},{"property" => "og:site_name", "content" => Site.first.title},{"property" => "og:url", "content" => request.original_url.split("?").first},{"property" => "og:description", "content" => meta_desc},{"property" => "og:image", "content" => "#{request.base_url}#{announcement.image.url}"},{"property" => "og:type", "content" => "Article"}])

    subtitle_ann = nil
    img_src = nil
    img_description = nil
    subtitle_ann = announcement.subtitle if announcement.display_subtitle?
    img_src = (announcement.image.thumb.url || "/assets/announcement-default.jpg") if announcement.display_img?
    img_description = announcement.image_description if (announcement.image_description.present?) && (announcement.display_img?)

    {
      "tags" => tags,
      "bulletin_files" => files,
      "bulletin_links" => links,
      "data" => {
        "title" => announcement.title,
        "subtitle_ann" => subtitle_ann,
        "update_user" => update_user,
        "updated_at" => announcement.postdate.strftime('%Y-%m-%d %H:%M'),
        "body" =>announcement.text,
        "image" => announcement.image.url,
        "img_src" => img_src,
        "img_description" => img_description,
        "alt_title" => desc
      },
      "impressionist" => (announcement.is_preview ? nil : announcement),
      "url_to_edit"=>url_to_edit
    }
  end

  def show_feed_announcement(uid)
    announcement = OrbitHelper.get_from_feed(uid)
    locale = OrbitHelper.get_site_locale.to_s
    url_to_edit = "#"
    return {} if announcement.blank?
    tags = []

    announcement["tags"].each{|tag|
      t = Tag.where(:name => tag["name_translations"][locale]).first rescue nil
      if t.nil?
        I18n.locale = (locale == "en" ? :zh_tw : :en)
        t = Tag.where(:name => tag["name_translations"][locale]).first rescue nil
        I18n.locale = locale.to_sym
      end
      tags << {
        "tag" => tag["name_translations"][locale],
        "url" => (t.nil? ?  "#" : OrbitHelper.page_for_tag(t))
      }
    }

    files = announcement["bulletin_files"].map{|file|  { "file_url" =>  file["url"], "file_title" => (file["title_translations"][locale] == "" ? URI.unescape(File.basename(file["url"])) : file["title_translations"][locale] rescue '') } } rescue []

    files.each do |file|
      if file["file_url"] =="" || file["file_url"] == nil
        files.delete(file)
      end
    end
    links = announcement["bulletin_links"].map{|link|  { "link_url" => link["url"], "link_title" => (link["title_translations"][locale] == "" ? link["url"] : link["title_translations"][locale]) } } rescue []

    update_user = announcement["author"]
    desc = announcement["image_description_translations"][locale] rescue ""
    desc = (desc.nil? || desc == "" ? "announcement image" : desc)

    request = OrbitHelper.request
    if announcement["subtitle_translations"].present?
      meta_desc = announcement["subtitle_translations"][locale] != "" ? announcement["subtitle_translations"][locale] : announcement["text_translations"][locale][0..200] rescue ""
    else
      meta_desc = ""
    end

    OrbitHelper.render_meta_tags([{"property" => "og:title", "content" => announcement["title_translations"][locale]},{"property" => "og:site_name", "content" => Site.first.title},{"property" => "og:url", "content" => request.original_url.split("?").first},{"property" => "og:description", "content" => meta_desc},{"property" => "og:image", "content" => announcement["image"]["original"]},{"property" => "og:type", "content" => "Article"}])

    datetime = DateTime.parse(announcement["postdate"])

    {
      "tags" => tags,
      "bulletin_files" => files,
      "bulletin_links" => links,
      "data" => {
        "title" => announcement["title_translations"][locale],
        "update_user" => update_user,
        "updated_at" => datetime.strftime('%Y-%m-%d %H:%M'),
        "body" => announcement["text_translations"][locale],
        "image" => announcement["image"]["original"],
        "alt_title" => desc
      },
      "impressionist" => nil,
      "url_to_edit" => url_to_edit
    }
  end

  def show
    params = OrbitHelper.params
    uid = params[:uid]
    if OrbitHelper.is_object_from_feed?(uid)
      show_feed_announcement(uid)
    else
      show_local_announcement(uid, (params["preview"] == "true" ? true : false))
    end
  end
  def show_widget
    @type = "show_widget"
    @show_page = params[:show_page]
    if params[:tags].nil?
      @tags = ['all']
    else
      @tags = params[:tags]
    end
    if params[:categories].nil?
      @categories = ['all']
    else
      @categories = params[:categories]
    end
    OrbitHelper.set_current_widget_module("announcement")
    OrbitHelper.set_params(params,current_user)
    Bulletin.remove_expired_status
    OrbitHelper.set_page_number(params[:page_no].to_i)
    OrbitHelper.set_page_data_count((params[:data_count].blank? ? 10 : params[:data_count].to_i))
    sorted,total_pages = get_sorted_annc
    anns = sorted.collect do |a|
      if a["source-site"].blank?
        statuses = a.statuses_with_classname.collect do |status|
          {
            "status" => status["name"],
            "status-class" => "status-#{status['classname']}"
          }
        end
        locale = I18n.locale.to_s
        files = a.bulletin_files.map{|file|  { "file_url" =>  file.file.url, "file_title" => (file.title.blank? ? File.basename(file.file.path) : file.title rescue '') } if file.enabled_for?(locale) } rescue []
        files.delete(nil)
        links = a.bulletin_links.map{|link|  { "link_url" => link.url, "link_title" => (link.title.blank? ? link.url : link.title) } } rescue []
        author = User.find(a.create_user_id).member_profile.name rescue ""
        desc = a.image_description
        desc = (desc.blank? ? "announcement image" : desc)
        link_to_show = (a.is_external_link ? a.external_link :  OrbitHelper.url_to_show(a.to_param)) rescue ""
        target = a.is_external_link ? "_blank" : "_self"
        doc = Nokogiri::HTML(a.title)
        title = doc.text.empty? ? 'no content' : doc.text
        {
          "department" => author,
          "bulletin_links" => links,
          "bulletin_files" => files,
          "title" => a.title,
          "source-site" => "",
          "source-site-title" => "",
          "source-site-link" => "",
          "subtitle" => a.subtitle,
          "statuses" => statuses,
          "category" => a.category.title,
          "postdate" => a.postdate,
          "author" => author,
          "is_top" => (a.is_top? ? 1 : 0),
          "link_to_show" => link_to_show+"\" title=\"#{title}\"",
          "target" => target,
          "img_src" => a.image.thumb.url || "/assets/announcement-default.jpg",
          "img_description" => desc,
          "more" => t(:more_plus),
          "view_count" => a.view_count
        }
      else
        a
      end
    end
    #If no data , hide title&table
    if sorted.count == 0
      display = "hide"
    end
    # anns = anns.concat(feeds_anns)
    # total_pages = announcements.total_pages


    @data = {
      "announcements" => anns,
      "extras" => {
        "widget-title" =>t('announcement.announcement'),
        "title-head" => t('announcement.table.title'),
        "date-head" => t('announcement.table.date'),
        "status-head" => t('announcement.table.status'),
        "author-head" => t('announcement.table.author'),
        "subtitle-head" => t('announcement.table.sub_title'),
        "category-head" => t('announcement.table.category'),
        "link-head" => t('announcement.table.link'),
        "file-head" => t('announcement.table.file'),
        "view-count-head" => t('announcement.table.view_count'),
        "display" => display,
        "department-head" => t('announcement.table.department')
      },
      "total_pages" => total_pages
    }
    render :layout => false
  end

end
